/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_main.h"
#include "../../src/initng_active_state.h"
#include "../../src/initng_static_states.h"

#include "initng_critical.h"

s_entry CRITICAL = { "critical", SET, NULL,
    "If this option is set, and service doesn't suceed, initng will quit and offer a sulogin."
};

/* returns TRUE if all use deps are started */
static int check_critical(active_db_h * service, a_state_h * status)
{
    assert(service);
    assert(service->name);

    /* if critial is set */
    if (!initng_active_db_is(&CRITICAL, service))
        return (TRUE);


    F_("Service %s failed, this is critical, going su_login!!\n",
       service->name);

    initng_main_su_login();
    /* now try to reload service from disk and run it again */

    if (service->from_service)
    {
        list_del(&(service->from_service->list));
        initng_service_cache_free(service->from_service);
        service->from_service = NULL;
    }
    initng_common_get_service(service);
    initng_common_mark_service(service, &WAITING_FOR_START_DEP);

    /* Make sure full runlevel starting fine */
    if (!initng_active_db_find_by_exact_name(g.runlevel))
        if (!initng_handler_start_new_service_named(g.runlevel))
            F_("runlevel \"%s\" could not be executed!\n", g.runlevel);

    return (FALSE);
}

int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module is compiled for \"%s\" version and initng is compiled on \"%s\" version, won't load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_service_data_types_add(&CRITICAL);
    initng_active_state_add_test(&FAIL_STARTING, 80, &check_critical);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_service_data_types_del(&CRITICAL);
    initng_active_state_del_test(&FAIL_STARTING, &check_critical);
}
