
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"
#define _GNU_SOURCE
#include <stdio.h>
#include <alloca.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/types.h>
#include <dirent.h>
/*#include <dirent.h> */     /* opendir() closedir() */

#include "../../src/initng_global.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_open_read_close.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_common.h"
#include "../../initng-paths.h"

#include "initng_find.h"

static char *get_find_alias(const char *from);

static service_cache_h *search_dir(const char *from, const char *dir)
{
    service_cache_h *tmp = NULL;
    DIR *path;
    struct dirent *dir_e;
    char t[100];
    char d[30];

    sprintf(d, "%s/%s", INITNG_ROOT, dir);

    path = opendir(d);
    if (!path)
        return (NULL);

    while ((dir_e = readdir(path)))
    {
        if (!strstr(dir_e->d_name, from))
            continue;

        strncpy(t, dir, 40);
        strcat(t, "/");
        strncat(t, from, 40);
        W_("Service should be %s/%s\n", dir, from);
        if ((tmp = initng_common_parse_service(t)))
        {
            closedir(path);
            return (tmp);
        }
    }
    closedir(path);
    return (NULL);
}

/* Load a service from a process_name or process_path */
service_cache_h *initng_find(const char *service)
{
    /* means that it may not be a direct service request, then we start looking in subfolders */

    service_cache_h *tmp = NULL;
    char *tmp_serv = NULL;

    assert(service);

    /* Never try to find a service with a '/' in the name, it alredy have a path */
    {
        int i = 0;

        while (service[i])
        {
            if (service[i] == '/')
                return (NULL);
            i++;
        }
    }

    if ((tmp = search_dir(service, "system")))
        return (tmp);

    if ((tmp = search_dir(service, "daemon")))
        return (tmp);

    /* try get from alias file */
    if (!(tmp_serv = get_find_alias(service))) ;
    return (NULL);

    if ((tmp = initng_common_parse_service(tmp_serv)))
    {
        free(tmp_serv);
        return (tmp);
    }

    free(tmp_serv);
    return (NULL);
}

#define ALIAS_FILE "/etc/initng/service_alias"
static char *get_find_alias(const char *from)
{
    int from_len = 0;
    int i = 0;

    char *file_content = NULL;
    char *point = NULL;
    char *point_result = NULL;
    char *ret = NULL;

    printf("Finding alias for %s\n", from);
    /* open that file */
    if (!open_read_close(ALIAS_FILE, &file_content))
        return (NULL);

    /* get lengt of what we are searching */
    from_len = strlen(from);

    /* Start Parsing */
    point = file_content;
    while (point[0])
    {
        /* skip initzial spaces */
        JUMP_SPACES(point);
        if (!point[0])
            break;

        /* skip rows, starting with '#' */
        if (point[0] == '#')
        {
            JUMP_TO_NEXT_ROW(point);
            continue;
        }

        printf("parse row \"%s\"\n", point);
        /* chek if first word is a match */
        if (strncasecmp(point, from, from_len) != 0)
        {
            printf("Not the one");
            JUMP_TO_NEXT_ROW(point);
            continue;
        }

        printf("point[from_len]='%c', point[from_len+1]='%c'\n",
               point[from_len], point[from_len + 1]);


        if (!point[from_len] || !point[from_len + 1]
            || point[from_len] != '=')
        {
            JUMP_TO_NEXT_ROW(point);
            continue;
        }

        /* Put pointer to result after the '=' */
        point_result = point + from_len + 1;

        while (point_result[i] && point_result[i] != '\n')
            i++;

        /* copy the rest of line */
        ret = i_strndup(point_result, i);

        free(file_content);
        printf("ret=\"%s\", len=%i\n", ret, i);
        return (ret);
    }
    free(file_content);
    return (NULL);
}



int module_init(const char *version)
{

    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module is compiled for \"%s\" version and initng is compiled on \"%s\" version, won't load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    return (initng_plugin_hook_add(PARSERS, 70, &initng_find));

}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_plugin_hook_del(PARSERS, &initng_find);
}
