/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"
#include "../../src/initng_static_states.h"

#include "initng_netprobe.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/ip.h>
#include <netinet/in.h>
#include <net/if.h>
#include <string.h>
#include <arpa/inet.h>

static int is_network(void);
static int check_START_DEP_MET(active_db_h * service);
static int check_STOP_DEP_MET(active_db_h * service);

int network_status = FALSE;
time_t last_check = 0;

static int is_network(void)
{
    struct ifconf ifc;
    struct ifreq *ifr;
    char buffert[1024];
    int netsock = -1;


    netsock = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (netsock < 0)
    {
        F_("Unable to open a socket!\n");
        return (FALSE);
    }

    ifc.ifc_len = sizeof(buffert);
    ifc.ifc_buf = buffert;

    if (ioctl(netsock, SIOCGIFCONF, &ifc) < 0)
    {
        F_("error: SIOCGIFCONF\n");
        return (FALSE);
    }

    /*printf("len:%i\n", ifc.ifc_len); */


    ifr = ifc.ifc_req;

    {
        int i;
        for (i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++)
        {
            /* Loopback is not network */
            if (strcmp("lo", ifr->ifr_name) == 0)
                continue;
            /*struct ifreq i; */
            D_("found up interface %s.\n", ifr->ifr_name);
            close(netsock);
            network_status = TRUE;
            return (TRUE);
        }

    }

    close(netsock);
    network_status = FALSE;
    return (FALSE);
}

s_entry REQUIRE_NETWORK = { "require_network", SET, NULL,
    "If set, initng will wait for an active internet connection before starting this service."
};
s_entry NETWORK_PROVIDER = { "network_provider", SET, NULL,
    "Set this on services that provides network, and if will be stopped after daemons have quit."
};


static int check_STOP_DEP_MET(active_db_h * service)
{
    active_db_h *current = NULL;

    /* The network provider should always be stoppable */
    if (initng_active_db_is(&NETWORK_PROVIDER, service))
        return (TRUE);

    /*
     * if there exits any service with NETWORK_PROVIDER set,
     * and it is not down, continue waiting for STOP_DEP
     */
    while_active_db(current)
    {
        if (initng_active_db_is(&NETWORK_PROVIDER, service)
            && (!IS_DOWN(service)))
            return (FALSE);
    }
    return (TRUE);
}

static int check_START_DEP_MET(active_db_h * service)
{
    /* only apply if service requires network */
    if (!initng_active_db_is(&REQUIRE_NETWORK, service))
        return (TRUE);

    /*
     * I belive that we are way better probing this in kernel, 
     * then waiting for the net provider service to be up.
     */
#ifdef REQUIRE_NETWORK_PROVIDER_SERVICE_TO_BE_UP
    active_db_h *current = NULL;

    while_active_db(current)
    {
        if (initng_active_db_is(&NETWORK_PROVIDER, current) && IS_UP(current))
            goto ok;
    }
    /* Dont set sleep 1, becouse initng will be interrupted when a
     * service changes state anyway.
     */
    /*initng_set_sleep(1); */
    return (FALSE);

  ok:
#endif

    /*
     * Now when we now that there is a network provider that
     * is really up, wo do an check with kernel network subsytem
     * so we now this is true.
     */

    /* only do this check once every mainloop */
    if (last_check != g.now.tv_sec)
    {
        network_status = is_network();
    }
    last_check = g.now.tv_sec;

    if (network_status == TRUE)
        return (TRUE);

    /* Make sure mainlop will run within 1 second. */
    initng_global_set_sleep(1);

    return (FALSE);
}


int module_init(const char *version)
{
    S_;
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module is compiled for \"%s\" version and initng is compiled on \"%s\" version, won't load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_service_data_types_add(&REQUIRE_NETWORK);
    initng_service_data_types_add(&NETWORK_PROVIDER);
    initng_active_state_add_test(&START_DEP_MET, 55, &check_START_DEP_MET);
    initng_active_state_add_test(&STOP_DEP_MET, 55, &check_STOP_DEP_MET);
    return (TRUE);
}

void module_unload(void)
{
    S_;
    initng_service_data_types_del(&REQUIRE_NETWORK);
    initng_service_data_types_del(&NETWORK_PROVIDER);
    initng_active_state_del_test(&START_DEP_MET, &check_START_DEP_MET);
    initng_active_state_del_test(&STOP_DEP_MET, &check_STOP_DEP_MET);
}
