
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"


#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <errno.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"


#include "initng_renice.h"
s_entry NICE = { "nice", INT, NULL, "Set this nice value before executing service." };

int do_renice(active_db_h * s, process_h * p __attribute__ ((unused)))
{
    assert(s);
    assert(s->name);
    assert(p);

    if (initng_active_db_is(&NICE, s))
    {
        D_("Will renice %s to %i !\n", s->name,
           initng_active_db_get_int(&NICE, s));
        errno = 0;
        if (nice(initng_active_db_get_int(&NICE, s)) == -1 && errno != 0)
        {
            F_("Failed to set the nice value: %s\n", strerror(errno));
            return FALSE;
        }
    }
    return (TRUE);
}

int module_init(const char *version)
{

    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module is compiled for \"%s\" version and initng is compiled on \"%s\" version, won't load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }
    D_("module_init();\n");
    initng_service_data_types_add(&NICE);
    return (initng_plugin_hook_add(A_FORK, 50, &do_renice));
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_service_data_types_del(&NICE);
    initng_plugin_hook_del(A_FORK, &do_renice);
}
