
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"
#define _GNU_SOURCE
#include <stdio.h>
#include <alloca.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <assert.h>

#include "../../src/initng_global.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_open_read_close.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_handler.h"
#include "../../src/initng_service.h"
#include "../../initng-paths.h"

#include "initng_rl_parser.h"

/* a simple parser for a runlevel file */
service_cache_h *initng_rl_parser(const char *runlevel_name)
{
    service_cache_h *n_service; /* service struct pointer too   */
    char *filetoparse = NULL;
    char *w_depends = NULL;
    char *w = NULL;
    char *a = NULL;

    assert(runlevel_name);

    filetoparse = (char *) i_calloc(strlen(INITNG_ROOT) + 1 +
                                    strlen(runlevel_name) + 10, sizeof(char));

    D_("initng_rl_parser(%s);\n", runlevel_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, runlevel_name);
    strcat(filetoparse, ".runlevel");
    D_("initng_rl_parser(%s): loading file %s.\n",
       runlevel_name, filetoparse);

    /* allocate a new service */
    if (!
        (n_service = initng_service_cache_new(runlevel_name, &TYPE_RUNLEVEL)))
    {
        free(filetoparse);
        return (NULL);
    }

    /* copy file filename into allocated file_buf */
    if (!open_read_close(filetoparse, &w_depends))
    {
        D_("parse_file(%s): Can't open config file!\n", filetoparse);
        if (n_service->name)
            free(n_service->name);

        /* free all data entrys */
        initng_service_cache_remove_all(n_service);

        if (n_service)
            free(n_service);
        free(filetoparse);
        return (NULL);
    }
    w = w_depends;

    while (w[0] != '\0')
    {
        /* skip leading spaces */
        JUMP_SPACES(w);
        if (w[0] == '\0')
            break;

        /* skip lines, starting with '#' */
        if (w[0] == '#')
        {
            JUMP_TO_NEXT_ROW(w);
            continue;
        }

        a = st_dup_line(&w);
        if (a)
        {
            D_("adding dep: \"%s\"\n", a);
            inting_service_cache_set_another_string(&NEED, n_service, a);
        }

        JUMP_TO_NEXT_ROW(w);
    }

    free(w_depends);

    initng_service_cache_set_string(&FROM_FILE, n_service,
                                    i_strdup(filetoparse));

    free(filetoparse);

    if (initng_service_cache_add(n_service))
        return (n_service);
    return (NULL);
}

int module_init(const char *version)
{
    D_("initng_rl_parser: module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module is compiled for \"%s\" version and initng is compiled on \"%s\" version, won't load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    return (initng_plugin_hook_add(PARSERS, 60, &initng_rl_parser));
}

void module_unload(void)
{
    initng_plugin_hook_del(PARSERS, &initng_rl_parser);
}
