/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <string.h>                         /* strstr() */
#include <stdlib.h>                         /* free() exit() */
#include <assert.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_states.h"

#include "initng_syncron.h"

s_entry SYNCRON = { "syncron", STRING, NULL,
    "All services with this same syncron string, can't be started asyncronous."
};

static int check;

/* Make sure if service syncron=module_loading, only one of the services with module_loading runs at once */
static int check_syncronicly_service(active_db_h * service,
                                     a_state_h * status)
{
    active_db_h *current, *q = NULL;
    const char *service_syncron;
    const char *current_syncron;

    assert(service);
    assert(service->name);

    if ((service_syncron =
         initng_active_db_get_string(&SYNCRON, service)) == NULL)
        return (TRUE);

    while_active_db_safe(current, q)
    {
        /* don't check ourself */
        if (current == service)
        {
            continue;
        }
        /* Check every other service in STARTING, see if it has SYNCRON = same_as_us */
        if (IS_MARK(current, &STARTING))
        {
            if ((current_syncron =
                 initng_active_db_get_string(&SYNCRON, current)) != NULL)
            {
                if (strcmp(service_syncron, current_syncron) == 0)
                {
                    D_("Service %s has to wait for %s\n", service->name,
                       current->name);
                    /* refuse to change status */
                    return (FALSE);
                }
            }
        }
    }
    return (TRUE);
}

/* Make sure there is only one service starting */
static int check_syncronicly(active_db_h * service, a_state_h * status)
{
    active_db_h *current, *q = NULL;

    while_active_db_safe(current, q)
    {
        /* don't check ourself */
        if (current == service)
            continue;
        if (current->current_state == &START_DEP_MET
            || current->current_state == &STARTING)
        {
            /* no i cant set this status yet */
            return (FALSE);
        }
    }
    return (TRUE);
}


int module_init(const char *version)
{
    int i;

    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module is compiled for \"%s\" version and initng is compiled on \"%s\" version, won't load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }
    initng_service_data_types_add(&SYNCRON);
    for (i = 0; g.Argv[i]; i++)
        if (strstr(g.Argv[i], "syncronicly"))
        {
            check = TRUE;
            initng_active_state_add_test(&START_DEP_MET, 10,
                                         &check_syncronicly);
            return (TRUE);
        }
    check = FALSE;
    /* Notice this is only added if we don't have --syncronicly */
    D_("Adding syncron\n");
    initng_active_state_add_test(&START_DEP_MET, 10,
                                 &check_syncronicly_service);


    return (TRUE);
}

void module_unload(void)
{

    initng_active_state_del_test(&START_DEP_MET, &check_syncronicly);
    initng_active_state_del_test(&START_DEP_MET, &check_syncronicly_service);
    initng_service_data_types_del(&SYNCRON);
}
