
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <termios.h>
#include <stdio.h>
#include <sys/klog.h>

#include "initng_global.h"
#include "initng_signal.h"
#include "initng_handler.h"
#include "initng_execute.h"
#include "initng_active_db.h"
#include "initng_load_module.h"
#include "initng_plugin_callers.h"
#include "initng_toolbox.h"


#include "initng_main.h"
/*extern char **environ; */

static int local_sulogin_count = 0;

/*
 * Poll if there are any processes left, and if not quit initng
 */
int initng_main_ready_to_quit(void)
{
    active_db_h *current = NULL;

    /* If the last process has died, quit initng */
    while_active_db(current)
    {
        /* Dont check with failed services */
        if (IS_FAILED(current))
            continue;

        /*
         * if we got here,
         * its must be an non failing service, 
         * left.
         */
        return (FALSE);
    }

    return (TRUE);
}

/* this is called when there is no processses left */
void initng_main_when_out(void)
{
    if (g.i_am_init && getpid() != 1)
    {
        F_("I AM NOT INIT, THIS CANT BE HAPPENING!\n");
        sleep(3);
        return;
    }

    switch (g.when_out)
    {
        case THEN_QUIT:
            P_(" ** Now Quiting **\n");
            initng_main_exit(0);
            break;
        case THEN_SULOGIN:
            P_(" ** Now SuLogin\n");
            initng_main_su_login();
            break;
        case THEN_RESTART:
            P_(" ** Now restarting\n");
            initng_main_restart();
            break;
        case THEN_NEW_INIT:
            P_(" ** Launching new init\n");
            initng_main_new_init();
            break;
        case THEN_REBOOT:
            P_(" ** Now rebooting\n");
            initng_main_exec_reboot();
            break;
        case THEN_HALT:
            P_(" ** Now halting\n");
            initng_main_exec_halt();
            break;
    }

}


#define TRY_TIMES 2
/*
 *  Launch sulogin and wait it for finish
 */
void initng_main_su_login(void)
{
    pid_t sulogin_pid;
    int status;

    /* sulogin nicely 2 times */
    if (local_sulogin_count <= TRY_TIMES)
    {
        printf("This is a sulogin offer,\n"
               "you will be able to login for %i times (now %i),\n"
               "and on return initng will try continue where it was,\n"
               "if the times go out, initng will launch\n"
               "/sbin/initng-segfault on next su_login request.\n\n",
               TRY_TIMES, local_sulogin_count);
        sulogin_pid = fork();

        if (sulogin_pid == 0)
        {
            char *argv[] = { i_strdup("sulogin"), NULL };

            execve("/sbin/sulogin", argv, environ);     /* launch sulogin */

            printf("Unable to execute /sbin/sulogin!\n");
            initng_main_segfault();
            _exit(1);
        }

        if (sulogin_pid > 0)
        {
            do
            {
                sulogin_pid = waitpid(sulogin_pid, &status, WUNTRACED);
            }
            while (!WIFEXITED(status) && !WIFSIGNALED(status));

            /* increase the sulogin_count */
            local_sulogin_count++;
            return;
        }
    }

    /* else run segfault script */
    initng_main_segfault();

    /* newer get here */
    _exit(1);
}

/*
 * Start initial services
 */
void initng_main_start_extra_services(void)
{
    int i;
    int a_count = 0;            /* counts orders from argv to start */

    initng_main_set_sys_state(STATE_STARTING);
    /* check with argv which service to start initiating */
    for (i = 1; i < g.Argc; i++)
    {
        /* start all services that is +service */
        if (g.Argv[i][0] != '+')
            continue;

        /* if succeed to load this service */
        if (initng_handler_start_new_service_named(g.Argv[i] + 1))
            a_count++;
        else
            F_(" Requested service \"%s\", could not be executed!\n",
               g.Argv[i]);
    }
}

/* This is same as execve() */
void initng_main_new_init(void)
{
    int i;

    initng_main_set_sys_state(STATE_EXECVE);
    for (i = 3; i <= 1013; i++)
    {
        close(i);
    }
    if (!g.new_init || !g.new_init[0])
    {
        F_(" g.new_init is not set!\n");
        return;
    }
    P_("\n\n\n          Launching new init (%s)\n\n", g.new_init[0]);
    execve(g.new_init[0], g.new_init, environ);
}

/* execve() reboot script */
void initng_main_exec_reboot(void)
{
    char **argv = NULL;
    int i;

    initng_main_set_sys_state(STATE_REBOOT);
    for (i = 3; i <= 1013; i++)
    {
        close(i);
    }
    argv = (char **) i_calloc(3, sizeof(char *));
    argv[0] = i_strdup("/sbin/shutdown_script");
    argv[1] = i_strdup("reboot");

    P_("\n\n\n          Launching /sbin/shutdown_script (reboot)\n");
    execve(argv[0], argv, environ);
}

/* execve() halt script */
void initng_main_exec_halt(void)
{
    char **argv = NULL;
    int i;

    initng_main_set_sys_state(STATE_HALT);

    for (i = 3; i <= 1013; i++)
    {
        close(i);
    }
    argv = (char **) i_calloc(3, sizeof(char *));
    argv[0] = i_strdup("/sbin/shutdown_script");
    argv[1] = i_strdup("poweroff");

    P_("\n\n\n          Launching /sbin/shutdown_script (poweroff)\n");
    execve(argv[0], argv, environ);
}


void initng_main_restart(void)
{
    char **argv = NULL;
    int i;

    initng_main_set_sys_state(STATE_RESTART);
    for (i = 3; i <= 1013; i++)
    {
        close(i);
    }
    argv = (char **) i_calloc(3, sizeof(char *));

    argv[0] = (char *) i_calloc(strlen(g.runlevel) + 12, sizeof(char));
    strcpy(argv[0], "runlevel=");
    strcat(argv[0], g.runlevel);
    argv[1] = NULL;
    P_("\n\n\n          R E S T A R T I N G,  (Really hot reboot)\n\n");
    execve("/sbin/initng", argv, environ);
}

/* this function sets g.sys_state, and call plugins that listen on its change */
void initng_main_set_sys_state(h_sys_state state)
{
    /* don't set a state that is */
    if (state == g.sys_state)
        return;

    D_("set_sys_state(): %% Setting state to: %i %% \n", state);
    g.sys_state = state;

    /*
     * execute all functions in modules that want to
     * be executed when system state change occurs.
     */

    initng_plugin_callers_load_module_system_changed(state);
    return;
}


void initng_main_exit(int i)
{
    D_("exit_initng();\n");

    initng_main_set_sys_state(STATE_EXIT);

    initng_unload_module_unload_all();
    initng_global_free();
    exit(i);
}

/*
 * This is called on segfault,
 * DONT call any other funtion in initng from here, not even P_, F_, W_!
 */
void initng_main_segfault(void)
{
    char *argv[3];

    /* if this is not init */
    if (!g.i_am_init)
    {
        /* youst quit */
        _exit(99);
    }

#ifdef DEBUG
    /* if we exit the program before sulogin, we wont be able to gdb initng */
    /*su_login(); */
    printf("Initng segfaulted, will wait in 20 seconds for you to start a gdb, before execve(/sbin/initng-segfault);\n");
    while (1)
        sleep(20);
#endif

    /*
     * IF we close sockets, it gets problems, when reloading after a segfault.
     int i
     for (i = 3; i <= 1013; i++)
     {
     close(i);
     }
     */

    argv[0] = strdup("/sbin/initng-segfault");
    argv[1] = NULL;

    printf("Launching /sbin/initng-segfault, this is pid: %i\n", getpid());
    if (execve("/sbin/initng-segfault", argv, environ) == -1)
    {
        printf("/sbin/initng-segfault did not exist, will die!\n");
    }
    _exit(99);
}

void initng_main_set_runlevel(const char *runlevel)
{
    /* first free the old_runlevel */
    if (g.old_runlevel)
    {
        free(g.old_runlevel);
        g.old_runlevel = NULL;
    }

    /* then move the last new, to the old */
    if (g.runlevel)
    {
        g.old_runlevel = g.runlevel;
        g.runlevel = NULL;
    }

    /* and set the new */
    g.runlevel = i_strdup(runlevel);
}
